from ekp_sdk.services.limiter import Limiter
from ekp_sdk.services.rest_client import RestClient
from ast import literal_eval

class EtherscanService:
    def __init__(
        self,
        api_key,
        base_url,
        rest_client: RestClient
    ):
        self.api_key = api_key
        self.base_url = base_url
        self.rest_client = rest_client
        self.limiter = Limiter(250, 10)

    async def get_latest_block_number(self):
        url = f"{self.base_url}?module=proxy&action=eth_blockNumber&apikey={self.api_key}"
        result = await self.rest_client.get(url, lambda data, text: literal_eval(data["result"]), self.limiter)

        return result
    
    async def get_contract_name(self, address):
        url = f"{self.base_url}?module=contract&action=getsourcecode&address={address}&apikey={self.api_key}"

        result = await self.rest_client.get(url, lambda data, text: data["result"][0]["ContractName"], self.limiter)

        return result

    async def get_abi(self, address):
        url = f"{self.base_url}?module=contract&action=getabi&address={address}&apikey={self.api_key}"

        result = await self.rest_client.get(url, lambda data, text: data["result"], self.limiter)

        return result

    async def get_transactions(self, address, start_block, offset):

        url = f'{self.base_url}?module=account&action=txlist&address={address}&startblock={start_block}&page=1&offset={offset}&sort=asc&apiKey={self.api_key}'

        def fn(data, text):
            trans = data["result"]
            
            if (trans is None or not isinstance(trans, list)):
                print(f"🚨 {text}")
                raise Exception("Received None data from url")

            return trans

        result = await self.rest_client.get(url, fn, self.limiter)

        return result

    async def get_logs(self, address, start_block):

        url = f'{self.base_url}?module=logs&action=getLogs&address={address}&fromBlock={start_block}&toBlock=latest&apiKey={self.api_key}'

        def fn(data, text):
            trans = data["result"]
            
            if (trans is None or not isinstance(trans, list)):
                print(f"🚨 {text}")
                raise Exception("Received None data from url")

            return trans

        result = await self.rest_client.get(url, fn, self.limiter)

        return result



from ekp_sdk.services.cache_service import CacheService
from ekp_sdk.services.client_service import ClientService
from ekp_sdk.services.coingecko_service import CoingeckoService
from ekp_sdk.services.etherscan_service import EtherscanService
from ekp_sdk.services.redis_client import RedisClient
from ekp_sdk.services.rest_client import RestClient
from ekp_sdk.services.web3_service import Web3Service
from ekp_sdk.services.limiter import Limiter



import json

from ekp_sdk.services.redis_client import RedisClient


class CacheService:
    def __init__(
        self,
        redis_client: RedisClient
    ):
        self.redis_client = redis_client

    async def get(self, key):
        value = await self.redis_client.get(key)
        if (value is None):
            return None
        return json.loads(value)

    async def set(self, key, value, ex=None):
        if (value is not None):
            return await self.redis_client.set(key, json.dumps(value), ex=ex)

        return await self.redis_client.set(key, value)

    async def wrap(self, key, fn, ex=None):

        cache_value = await self.get(key)

        if (cache_value is not None):
            return cache_value

        value = await fn()

        await self.set(key, value)

        return value



from ekp_sdk.util.clean_null_terms import clean_null_terms


def Chart(
        type,
        series,
        title=None,
        card=None,
        data=None,
        height=400,
        busy_when=None,
        options=None,
        class_name=None
):
    return {
        "_type": "Chart",
        "props": clean_null_terms({
            "busyWhen": busy_when,
            "card": card,
            "className": class_name,
            "data": data,
            "height": height,
            "options": options,
            "series": series,
            "title": title,
            "type": type,
        })
    }


import json
import time

import aiohttp
from aioretry import retry
from ekp_sdk.services.limiter import Limiter
from ekp_sdk.util.retry import default_retry_policy


class RestClient:

    @retry(default_retry_policy)
    async def get(
        self,
        url,
        fn=lambda data, text: data,
        limiter: Limiter = None
    ):
        if limiter is not None:
            await limiter.acquire()
        
        try:
            async with aiohttp.ClientSession() as session:
                print(f"🐛 GET {url}")
                start = time.perf_counter()
                response = await session.get(url=url)

                if (response.status != 200):
                    raise Exception(f"Response code: {response.status}")

                text = await response.read()
                data = json.loads(text.decode())

                print(f"⏱  GET [{url}] {time.perf_counter() - start:0.3f}s")

                return fn(data, text)
        finally:
            if limiter is not None:
                limiter.release()

    async def post(
        self,
        url,
        data,
        fn=lambda data, text: data,
        limiter: Limiter = None
    ):
        if limiter is not None:
            await limiter.acquire()

        try:
            async with aiohttp.ClientSession() as session:
                print(f"🐛 POST {url}")
                start = time.perf_counter()
                response = await session.post(url=url, data=json.dumps(data), headers={"content-type": "application/json"})

                if (response.status not in [200, 201]):
                    raise Exception(f"Response code: {response.status}")

                text = await response.read()
                data = None
                if text:
                    data = json.loads(text.decode())

                print(f"⏱  POST [{url}] {time.perf_counter() - start:0.3f}s")

                return fn(data, text)
        finally:
            if limiter is not None:
                limiter.release()


from ekp_sdk.db.mg_client import MgClient

from ekp_sdk.db.pg_client import PgClient
from ekp_sdk.services.cache_service import CacheService
from ekp_sdk.services.client_service import ClientService
from ekp_sdk.services.coingecko_service import CoingeckoService
from ekp_sdk.services.etherscan_service import EtherscanService
from ekp_sdk.services.redis_client import RedisClient
from ekp_sdk.services.rest_client import RestClient
from ekp_sdk.services.web3_service import Web3Service



class BaseContainer:
    def __init__(self, config):

        EK_PLUGIN_ID = config("EK_PLUGIN_ID", default=None)
        ETHERSCAN_API_KEY = config("ETHERSCAN_API_KEY", default=None)
        ETHERSCAN_BASE_URL = config("ETHERSCAN_BASE_URL", default=None)
        MONGO_DB_NAME = config('MONGO_DB_NAME', default=None)
        MONGO_URI = config('MONGO_URI', default=None)
        PORT = config("PORT", default=3001, cast=int)
        POSTGRES_URI = config("POSTGRES_URI", default=None)
        REDIS_URI = config("REDIS_URI", default=None)
        WEB3_PROVIDER_URL = config("WEB3_PROVIDER_URL", default=None)

        self.rest_client = RestClient()

        if REDIS_URI is not None:
            self.redis_client = RedisClient(
                uri=REDIS_URI
            )
        else:
            print("⚠️ skipped RedisClient init, missing REDIS_URI")

        if POSTGRES_URI is not None:
            self.pg_client = PgClient(
                uri=POSTGRES_URI,
            )
        else:
            print("⚠️ skipped PgClient init, missing POSTGRES_URI")

        if MONGO_URI is not None:
            self.mg_client = MgClient(
                uri=MONGO_URI,
                db_name=MONGO_DB_NAME
            )
        else:
            print("⚠️ skipped MgClient init, missing MONGO_URI")

        if ETHERSCAN_API_KEY is not None and ETHERSCAN_BASE_URL is not None:
            self.etherscan_service = EtherscanService(
                api_key=ETHERSCAN_API_KEY,
                base_url=ETHERSCAN_BASE_URL,
                rest_client=self.rest_client
            )
        else:
            print(
                "⚠️ skipped EtherscanService init, missing ETHERSCAN_API_KEY and ETHERSCAN_BASE_URL")

        if REDIS_URI is not None:
            self.cache_service = CacheService(
                redis_client=self.redis_client,
            )
        else:
            print("⚠️ skipped CacheService init, missing REDIS_URI")

        if WEB3_PROVIDER_URL is not None:
            self.web3_service = Web3Service(
                provider_url=WEB3_PROVIDER_URL,
            )
        else:
            print("⚠️ skipped Web3Service init, missing WEB3_PROVIDER_URL")

        if EK_PLUGIN_ID is not None:
            self.client_service = ClientService(
                port=PORT,
                plugin_id=EK_PLUGIN_ID
            )
        else:
            print("⚠️ skipped ClientService init, missing EK_PLUGIN_ID")

        self.coingecko_service = CoingeckoService(
            rest_client=self.rest_client
        )




import json

from ekp_sdk.services.rest_client import RestClient


class CoingeckoService:
    def __init__(
        self,
        rest_client: RestClient
    ):
        self.base_url = "https://api.coingecko.com/api/v3"
        self.rest_client = rest_client

    async def get_coin_id_map(self, platform_id):

        url = f"{self.base_url}/coins/list?include_platform=true"

        response = await self.rest_client.get(url)

        map = {}

        for coin in response:
            if (platform_id not in coin["platforms"]):
                continue

            map[coin["platforms"][platform_id]] = coin["id"]

        return map

    async def get_historic_price(self, coin_id, date_str, fiat_id):

        url = f"{self.base_url}/coins/{coin_id}/history?date={date_str}"

        result = await self.rest_client.get(url, lambda data, text: data['market_data']['current_price'][fiat_id])

        return result

    async def get_latest_price(self, coin_id, fiat_id):

        url = f"{self.base_url}/simple/price?ids={coin_id}&vs_currencies={fiat_id}"

        result = await self.rest_client.get(url, lambda data, text: data[coin_id][fiat_id])

        return result



import asyncio

import aioredis


class RedisClient:
    def __init__(self, uri):
        self.r = aioredis.from_url(uri)

    async def get(self, key):
        return await self.r.get(key)

    async def set(self, key, value, ex=None):
        return await self.r.set(key, value, ex=ex)



from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

VERSION = '0.4.3'
DESCRIPTION = 'Python SDK for frontend components usage'
LONG_DESCRIPTION = 'A package that allows to use front-end components to simplify ' \
                   'building of web pages for ekp plugins'


def get_list_of_names(filename):
    list_of_names = []
    with open(filename, "r") as fh:
        for name in fh:
            list_of_names.append(name.replace('\n', ''))
    return list_of_names


# Setting up
setup(
    name="ekp-sdk",
    version=VERSION,
    url="https://github.com/earnkeeper/python-ekp-sdk",
    author="Earn Keeper (Gavin Shaw)",
    author_email="gavin@earnkeeper.io",
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=get_list_of_names('requirements.txt'),
    keywords=['python', 'earnkeeper', 'sdk', 'ekp'],
    data_files=['requirements.txt'],
    classifiers=[
        # "Development Status :: 1 - Planning",
        # "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
        # "Operating System :: Unix",
        # "Operating System :: MacOS :: MacOS X",
        # "Operating System :: Microsoft :: Windows",
    ]
)



# https://betterprogramming.pub/how-to-remove-null-none-values-from-a-dictionary-in-python-1bedf1aab5e4
def clean_null_terms(d):
    clean = {}
    for k, v in d.items():
        if isinstance(v, dict):
            nested = clean_null_terms(v)
            if len(nested.keys()) > 0:
                clean[k] = nested
        elif v is not None:
            clean[k] = v
    return clean